local addonName, addonData = ...;

-- Variables
local events = {};

local timeThisLevel = nil;
local totalTimePlayed = 0;
local lastPlayTimeRequest = 0;
local requestTimePlayed = true;

local function ShowXpTooltip()
	GameTooltip:ClearLines();
	
	local xpFormat, xpRestedFormat, xpPercentageNeeded, xpRestedPercentage, maxLevel = DLU.XpLeft();
	local currentLevel = UnitLevel("player");
	
	-- Change to max level later
	if (maxLevel > currentLevel) then
		local nextLevel = currentLevel + 1;
		
		
		GameTooltip:SetOwner(DLUHQ.Panels.Character.XpBar, 'ANCHOR_CURSOR', 0, -4);
		GameTooltip:AddLine(addonData.L.XP);
		GameTooltip:AddLine(" ");
		GameTooltip:AddDoubleLine(addonData.L.NextLevel, nextLevel .. " / ".. maxLevel,1,1,1);
		if (xpRestedFormat ~= nil) then
			local restedPercentage = format("%.2f", xpRestedPercentage);
			GameTooltip:AddDoubleLine(addonData.L.RestedXp, DLU.itemColorString(3, format("%s (%s%%)", xpRestedFormat, restedPercentage)), 1,1,1);
		end
		
		if (timeThisLevel ~= nil) then
			local thisLvlTime = time() + timeThisLevel - lastPlayTimeRequest;
			GameTooltip:AddDoubleLine("Time this lvl", DLU.GetFormattedTime(thisLvlTime),1,1,1);
		else
			GameTooltip:AddDoubleLine("Time this lvl", "Unknown...",1,1,1);
		end
		
		local totalTimePlayedCalc = time() + totalTimePlayed - lastPlayTimeRequest;
		GameTooltip:AddDoubleLine("Total time played", DLU.GetFormattedTime(totalTimePlayedCalc),1,1,1);
		
		GameTooltip:Show();
	end
end

local function updateXpUI()
	local currentLevel = UnitLevel("player");
	local xpFormat, xpRestedFormat, xpPercentageNeeded, xpRestedPercentage, maxLevel = DLU.XpLeft();
	
	--print(xpFormat, xpRestedFormat, xpPercentageNeeded, xpRestedPercentage);
	
	-- Change to max level later
	if (maxLevel > currentLevel) then
		local nextLevel = currentLevel + 1;
		
		local xpPercentGained = 100 - xpPercentageNeeded;
		
		DLUHQ.Panels.Character.XpBar:SetValue(xpPercentGained);
		DLUHQ.Panels.Character.XpBar.Label:SetText(format("%s (%i%%)", xpFormat, xpPercentageNeeded));
		
		if (xpRestedPercentage ~= nil) then
			if (xpRestedPercentage < 100) then
				DLUHQ.Panels.Character.XpBar.Rested:SetValue(xpPercentGained + xpRestedPercentage);
			else
				DLUHQ.Panels.Character.XpBar.Rested:SetValue(100);
			end
		else
			DLUHQ.Panels.Character.XpBar.Rested:SetValue(0);
		end
	else
		DLU.ChangeStatusBarColor(DLUHQ.Panels.Character.XpBar, {
			r = .3,
			g = .3,
			b = .3
		});
		DLUHQ.Panels.Character.XpBar:SetValue(100);
		DLUHQ.Panels.Character.XpBar.Label:SetText("Max level");
	end
end

local o = ChatFrame_DisplayTimePlayed;
ChatFrame_DisplayTimePlayed = function(...)
	if (requestTimePlayed) then
		requestTimePlayed = false;
		return;
	end
	
	return o(...);
end

local function UpdatePlayTime()
	requestTimePlayed = true;
	RequestTimePlayed();
end

--[[
function DLU.UpdateReputationUI()
	local name, reaction, Min, Max, value, factionID = GetWatchedFactionInfo();
	local color = _G.FACTION_BAR_COLORS[reaction];
	
	if (name ~= nil) then
		local standing = getglobal("FACTION_STANDING_LABEL".. reaction);

		if (not DLUHQ.Panels.Character.ReputationBar:IsVisible()) then
			DLUHQ.Panels.Character.ReputationBar:Show();
		end
		
		if (C_Reputation.IsFactionParagon(factionID)) then
			local currentValue, threshold, _, hasRewardPending = C_Reputation.GetFactionParagonInfo(factionID);
			if currentValue and threshold then
				Min, Max = 0, threshold
				value = currentValue % threshold
				if hasRewardPending then
					value = value + threshold
				end
			end
		end
		
		DLUHQ.Panels.Character.ReputationBar.statusBar:SetStatusBarColor(color.r, color.g, color.b, 1);
		DLUHQ.Panels.Character.ReputationBar.statusBar:SetMinMaxValues(Min, Max);
		DLUHQ.Panels.Character.ReputationBar.statusBar:SetValue(value);
		
		local MinVal = value - Min;
		local MaxVal = Max - Min;
		
		if (DLUSettings["repFormat"] == "Left (%left)") then
			local repLeft = MaxVal - MinVal;
			local currentPercentage = floor((repLeft / MaxVal) * 100);
			
			DLUHQ.Panels.Character.ReputationBar.text:SetText(format("%s (%s): %i (%i%%)", name, standing, repLeft, currentPercentage));
		else
			DLUHQ.Panels.Character.ReputationBar.text:SetText(format("%s (%s): %i / %i", name, standing, MinVal, MaxVal));
		end
	else
		DLUHQ.Panels.Character.ReputationBar:Hide();
	end
end
]]

local function CreateCharacterPanel()
	local width = DLU.GetWidth();
	local height = DLU.GetHeight();
	local padding = DLU.GetPaddings();
	local fonts = DLU.GetFontSizes();
	
	--- Character
	DLUHQ.Panels.Character = DLU.CreatePanel(width.DLUHQ, height.DLUHQ - height.Nav, DLUHQ.Container, -height.Nav);
	DLU.createTextRoot(addonData.L.Character, "TOP", DLUHQ.Panels.Character, 8, -6, "GameFontNormalLarge");
	
	-- XP (status bar)
	DLUHQ.Panels.Character.XpBar = DLU.CreateStatusBar(DLUHQ.Panels.Character, width.DLUHQ - padding.Normal, fonts.medium, "TOPLEFT", padding.Normal/2, -(5 + padding.Top), {
		r = .75,
		g = 0,
		b = .75
	}, ShowXpTooltip);
	
	DLUHQ.Panels.Character.XpBar.Rested = DLU.CreateStatusBar(DLUHQ.Panels.Character, width.DLUHQ - padding.Normal, fonts.medium, "TOPLEFT", padding.Normal/2, -(5 + padding.Top), {
		r = .25,
		g = .5,
		b = 1
	}, ShowXpTooltip);
	
	DLUHQ.Panels.Character.XpBar.border = DLU.CreateBorder(DLUHQ.Panels.Character.XpBar);
	DLUHQ.Panels.Character.XpBar.Label = DLU.CreateLabel("", fonts.medium-2, "CENTER", DLUHQ.Panels.Character.XpBar);
	updateXpUI();
end

-- Events
function events:UPDATE_EXHAUSTION()
	updateXpUI();
end
function events:PLAYER_XP_UPDATE()
	updateXpUI();
end
function events:UPDATE_EXPANSION_LEVEL()
	updateXpUI();
end
function events:MERCHANT_SHOW()
	if (CanMerchantRepair() and DLUCharacterSettings.autoRepair == true) then
		local cost, canRepair = GetRepairAllCost();
		
		if (canRepair) then
			if (cost <= GetMoney()) then

				local guildGoldLimit = GetGuildBankWithdrawGoldLimit();
				if (guildGoldLimit > 0) then
					guildGoldLimit = guildGoldLimit * 10000;
				end
				
				local repairText = "";
				local repairCostString = GetCoinTextureString(cost);
				
				if (guildGoldLimit == -1 or guildGoldLimit >= cost) then
					RepairAllItems(1);
					repairText = format("Repaired gear costing %s using guild repair.", repairCostString);
				else
					RepairAllItems(nil);
					repairText = format(addonData.L.CharacterRepairMsg, repairCostString);
				end
				
				DLU.createMessage(repairText, "");
			end
		end
	end
end
function events:PLAYER_LEVEL_UP(level, healthDelta, powerDelta, numNewTalents, numNewPvpTalentSlots, strengthDelta, agilityDelta, staminaDelta, intellectDelta)
	UpdatePlayTime();
end
function events:TIME_PLAYED_MSG(arg1, arg2)
	totalTimePlayed = arg1;
	timeThisLevel = arg2;
	lastPlayTimeRequest = time();
end

DLU.EventFrame:RegisterEvent("PLAYER_LOGIN");
DLU.EventFrame:SetScript("OnEvent", function()
	DLU.EventFrame:SetScript("OnEvent", function(self, event, ...)
		events[event](self, ...);
	end);
	for k, v in pairs(events) do
		DLU.EventFrame:RegisterEvent(k);
	end
	
	CreateCharacterPanel();
	
	UpdatePlayTime();
end);