local addonName, addonData = ...;

local events = {};
local f = CreateFrame("Frame");

local function IsFriendshipRep(factionID)
    local factionInfo = C_GossipInfo.GetFriendshipReputation(factionID);

    if (factionInfo ~= nil and factionInfo.name ~= nil) then
        return true, factionInfo;
    end

    return false, nil;
end

local function IsMajorFaction(factionID)
    if (C_Reputation.IsMajorFaction(factionID)) then
		local majorFactionData = C_MajorFactions.GetMajorFactionData(factionID);
        if (majorFactionData ~= nil and majorFactionData.name) then
            local hasMaxRenown = C_MajorFactions.HasMaximumRenown(factionID);
            return true, majorFactionData, hasMaxRenown;
        end
	end

    return false, nil;
end

function DLU.UpdateRepuationUI()
    local name, reaction, Min, Max, value, factionID = GetWatchedFactionInfo();
    if (name ~= nil) then
        if (not DLUHQ.Panels.Character.ReputationBar:IsVisible()) then
            DLUHQ.Panels.Character.ReputationBar:Show();
        end

        local color = _G.FACTION_BAR_COLORS[reaction];
        local isCapped = false;
	
        local isFriendship, factionInfo = IsFriendshipRep(factionID);
        local IsMajorFaction, majorFactionInfo, hasMaxRenown = IsMajorFaction(factionID);
        local barText = "%s (%s): %i / %i";
    
        if (isFriendship) then
            name = factionInfo.name;
            reaction = factionInfo.reaction;

            if (factionInfo.nextThreshold) then
                Min = factionInfo.reactionThreshold;
                Max = factionInfo.nextThreshold;
                value = factionInfo.standing;
            else
                isCapped = true;
            end
        elseif (IsMajorFaction) then
            name = majorFactionInfo.name;
            reaction = format(addonData.L.RenownLevelText, majorFactionInfo.renownLevel);
            Min = 0;
            Max = majorFactionInfo.renownLevelThreshold;
            value = majorFactionInfo.renownReputationEarned;

            if (hasMaxRenown) then
                isCapped = true;
            end
        else
            if (reaction == MAX_REPUTATION_REACTION) then
                isCapped = true;
            end
        end

        if (isCapped) then
            Min, Max, value = 0, 1, 1;
        end

        local maxText = Max;
        local valueText = value;
        if (not IsMajorFaction and not isCapped) then
            maxText = Max - Min;
            valueText = value - Min;
        elseif (isCapped) then
            barText = "%s (%s)";
        end

        DLUHQ.Panels.Character.ReputationBar.statusBar:SetStatusBarColor(color.r, color.g, color.b, 1);
		DLUHQ.Panels.Character.ReputationBar.statusBar:SetMinMaxValues(Min, Max);
		DLUHQ.Panels.Character.ReputationBar.statusBar:SetValue(value);

        if (type(reaction) == "number") then
            reaction = getglobal("FACTION_STANDING_LABEL".. reaction);
        end

        DLUHQ.Panels.Character.ReputationBar.text:SetText(format(barText, name, reaction, valueText, maxText));
    else
        DLUHQ.Panels.Character.ReputationBar:Hide();
    end
end

local function Reputation_OnClick()
	ToggleCharacter("ReputationFrame");
end

local function CreateReputationUI()
    local width = DLU.GetWidth();
	local height = DLU.GetHeight();
	local padding = DLU.GetPaddings();
	local fonts = DLU.GetFontSizes();
    
	DLUHQ.Panels.Character.ReputationBar = DLU.CreateBar(DLUHQ.Panels.Character, nil, Reputation_OnClick, width.DLUHQ - padding.Normal, fonts.medium + 2, "TOPLEFT", padding.Normal/2, -(5 + padding.Top + fonts.medium) - 6, {
		r = 1,
		g = 1,
		b = 1
	});
    DLU.UpdateRepuationUI();
end

function events:UPDATE_FACTION()
	DLU.UpdateRepuationUI();
end
function events:COMBAT_TEXT_UPDATE()
	DLU.UpdateRepuationUI();
end
function events:QUEST_REMOVED(questId)
	DLU.UpdateRepuationUI();
end
f:RegisterEvent("PLAYER_LOGIN");
f:SetScript("OnEvent", function()
	f:SetScript("OnEvent", function(self, event, ...)
		events[event](self, ...);
	end);
	for k, v in pairs(events) do
		f:RegisterEvent(k);
	end

    CreateReputationUI();
end);